;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Winders Studio
;;  Copyright (C) 2022 Kevin Serna García, Nerea Pérez Ayestarán, Javier José Botella Rico
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------



.include "game.h.s"
.include "cpctelera.h.s"
.include "cpctelerafuncs.h.s"

unavariable: .db #0x10
isr:
    ld a, (unavariable)
    dec a
    ld (unavariable), a
    ret nz

        call z, cpct_akp_musicPlay_asm
        ld a, #12
        ld (unavariable), a
    ret

man_game_init::
    call cpct_disableFirmware_asm
    ;;;;;;;;;; IA ;;;;;;;;;
    ;; Iniciar sistema de IA
    call _man_entity_getArray ;;aqui cargo en IX el sitio donde empiezan las entidades y en A el numero de entidades
    call sys_ai_control_init
    ;;;;;;;;;;;;;;;;;;;;;;;

    ;; Iniciamos render
    call _sys_render_init
 
    ;; Reservamos memoria y ponemos las posiciones a 0
    call _man_entity_init

    ;;Iniciamos musica (iniciar no darle play)
    ld de, #_song_menu
    call cpct_akp_musicInit_asm

    ;;Set ISR
    ld hl, #isr
    call cpct_setInterruptHandler_asm

    ld a,#0x02

    ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;NUEVA PARTIDA
man_game_new::
    call _sys_render_erase_background
    call _sys_render_background_inGame
    call _sys_render_score

    ld hl,#PLAYER
    call _createEntity
    ;ld hl,#ENEMY
    ;call _createEntity

    call _man_create_spawn
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;CUANDO MUERE EL JUGADOR -> RESETEAMOS
man_end_play::
    call _man_destroy_all_entities
    call _sys_render_update
    call _man_entity_update
    call _sys_render_erase_background
    call _sys_render_background_gameOver
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;BUCLE GAMEPLAY
man_loop_play::
    call sys_input_update
    ;;;;;;;;;; IA ;;;;;;;;;
    call _man_entity_getArray ;;aqui cargo en IX el sitio donde empiezan las entidades y en A el numero de entidades 
    call sys_ai_control_init
    ld e_ai_aim_x(ix), #1
    call sys_ai_control_update
    ;;;;;;;;;;;;;;;;;;;;;;;
    ;; Se actualizan las posiciones
    
    call _sys_physics_update
    call _sys_update_collision_enemies
    call cpct_waitVSYNC_asm
    call _sys_render_update
    
   
    
    call _man_entity_update
    
    call _man_update_timer
    
    call _man_all_spawn_timer
    call _man_all_spawn_generate_enemy
    
   
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;LOGICA DEL BUCLE DEL JUEGO
man_game_play::

    loop_game:
    
    sub a,#0x01
    jr nz,_out_of_loop
    
        call man_loop_play
        ;;Comprobamos si no quedan vidas para acabar juego
        ld a,#0x01
        ld ix,#_entidades
        ld a, e_vida(ix)
        cp #0
        jr nz,_cont_loop
            ;;Terminamos juego y saltamos a pantalla game over
            call man_end_play
 
            ld a,#0x03
            jr loop_game
    
    _cont_loop:
    ld a,#0x01
    
    jr loop_game

    _out_of_loop:

    sub a,#0x01
    jr nz,_game_over

    push af
    call _sys_render_background_menu
    
    pop af
    add a,#0x01
    call sys_input_update
    sub a,#0x01
    jr nz,_add_two
    
    add a,#0x01
    push af

    call man_game_new
    
    pop af
    

    jr loop_game

    _add_two:

    add a,#0x03

    jr loop_game

    _game_over:

    ;;Crear función en el render que renderice solo el fondo de game over y llamarla aqui
    add a,#0x01
    call sys_input_update

    cp #0
    jr z,_cont_game_over

    add a,#0x01

    jr loop_game

    _cont_game_over:

    add a,#0x03

    jr loop_game

    ret