;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Winders Studio
;;  Copyright (C) 2022 Kevin Serna García, Nerea Pérez Ayestarán, Javier José Botella Rico
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INCLUDES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 .include "render.h.s"
 .include "man/entity.h.s"
 .include "cpctelera.h.s"
 .include "cpctelerafuncs.h.s"
 .include "globales.h.s"
 .include "sprites/screens/screen_juego.h.s"
 .include "sprites/screens/screen_menu.h.s"
 .include "sprites/screens/screen_fin.h.s"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GLOBAL SYMBOLS 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DEFINED VALUES 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 screen_start = 0xC000
 separacion_vidas = SP_VIDA_W + 1
 string_score: .asciz "sc:" ;;Transforma en ascii y anyade 0 al final
;;Inicializamos render-> memoria de video, paleta, borde pantalla
_sys_render_init::
 ld c,  #0                      ;;Video Mode = 0 -> 2x2 bytes
 call cpct_setVideoMode_asm

 ld hl, #_g_palette             ;;Usamos nuestra paleta
 ld de, #16                     ;;Nº colores paleta
 call cpct_setPalette_asm
 cpctm_setBorder_asm HW_BLACK   ;;Color borde pantalla
 ;;call _sys_render_background    ;;Pinta fondo
 ;;call _sys_render_score          ;;Pintamos score
 ;;call _sys_change_screen_resolution
 ret


;;DESTROYS BC, DE, HL
_sys_render_erase_background::
  
    cpctm_clearScreen_asm #0x00
   
    ret

;;DESTROYS AF, BC, DE, HL
_sys_render_background_menu::
   
    ld hl, #_screen_menu_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm
   
    ret
;;DESTROYS AF, BC, DE, HL
_sys_render_background_inGame::
    ld hl, #_screen_juego_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm
    ret

_sys_render_background_gameOver::
   
    ld hl, #_screen_fin_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm
   
    ret

;;Render Vida: Pinta num vidas player
;;Input: IX -> Inicio entidades = player
_sys_render_vida::

    X = 1
    Y = 3
    ld b, #3   ;;cargamos num vidas en a
    jr z, loop_huecos
    ld a, #0           ;;Donde pintaremos vidas (width)
    
    loop_huecos:
        push bc                       ;;Guardamos bc (B = num vidas a pintar)
        ld de,#screen_start +  + 64 * (Y / 8) + 2048 * (Y & 7) + X                     
        ld c, a                       ;; Pos X -> C = A
        ld b, #0                      ;; Pos Y -> inicio pantalla
        add a, #separacion_vidas      ;; A = A + espacio vidas
        push af                       ;; Guardamos A
        call cpct_getScreenPtr_asm    ;; HL = pvmen

        ex de, hl
        ld hl, #_sp_vida_vacia    
        ld c, #SP_VIDA_W              ;; Entity Height
        ld b, #SP_VIDA_H              ;; Entity Width
        call cpct_drawSprite_asm    
        
        pop af                        ;; Recuperamos A
        pop bc                        ;; Recuperamos B
    djnz loop_huecos                   ;;Saltamos si B != 0


    ld b, e_vida(ix)   ;;cargamos num vidas en a
    jr z, loop_vidas
    ld a, #0           ;;Donde pintaremos vidas (width)
    
    loop_vidas:
        push bc                       ;;Guardamos bc (B = num vidas a pintar)
        ld de,#screen_start +  + 64 * (Y / 8) + 2048 * (Y & 7) + X                     
        ld c, a                       ;; Pos X -> C = A
        ld b, #0                      ;; Pos Y -> inicio pantalla
        add a, #separacion_vidas      ;; A = A + espacio vidas
        push af                       ;; Guardamos A
        call cpct_getScreenPtr_asm    ;; HL = pvmen
    
        ex de, hl
        ld hl, #_sp_vida    
        ld c, #SP_VIDA_W              ;; Entity Height
        ld b, #SP_VIDA_H              ;; Entity Width
        call cpct_drawSprite_asm   
        
        pop af                        ;; Recuperamos A
        pop bc                        ;; Recuperamos B
    djnz loop_vidas                   ;;Saltamos si B != 0
    


    ret


;;Render Score: Pinta letras puntuacion y score player
_sys_render_score::

    ;;COLOR CHAR
    ;;(1B L ) fg_pen	Foreground palette colour index (Similar to BASIC’s PEN, 0-15)
    ;;(1B H ) bg_pen	Background palette colour index (PEN, 0-15)
    ld l, #1
    ld h, #3
    call cpct_setDrawCharM0_asm

    ;;Pintamos string "score: "
    ld iy, #string_score
   
    X = 59
    Y = 4
    ld hl, #0xC000 + 64 * (Y / 8) + 2048 * (Y & 7) + X
    call cpct_drawStringM0_asm
    ;;Pintamos la puntuacion del jugador
    ;;ld a, e_score(ix)

    ret

;;Input IX -> primera entidad (player)
_sys_render_player_score::
    ;;COLOR CHAR
    ;;(1B L ) fg_pen	Foreground palette colour index (Similar to BASIC’s PEN, 0-15)
    ;;(1B H ) bg_pen	Background palette colour index (PEN, 0-15)

    ld l, #1
    ld h, #3
    call cpct_setDrawCharM0_asm
    
    call _sys_decimalToAscii_digit_0
    call _sys_decimalToAscii_digit_1
    
    ret

_sys_decimalToAscii_digit_0::
    push ix
    ld a, e_puntuacion(ix)  ;;PUNTUACION = 00 -> tomamos primer digito
    rra
    rra
    rra
    rra
    and #0x0F
    
    add #48
    
    X = 71
    Y = 5
    ld hl, #0xC000 + 64 * (Y / 8) + 2048 * (Y & 7) + X
    ld e, a
    call cpct_drawCharM0_asm  
    pop ix
    ret

_sys_decimalToAscii_digit_1::
    push ix
    ld a, e_puntuacion(ix)  ;;PUNTUACION = 00 -> tomamos segundo digito
    and #0x0F
    add #48

    X = 75
    Y = 5
    ld hl, #0xC000 + 64 * (Y / 8) + 2048 * (Y & 7) + X  
    ld e, a
    call cpct_drawCharM0_asm        ;;DESTROYS 
    pop ix
    ret
;;Le pasamos por parámetro un Entity_t* e
;;Nos llega el e en DE
;;INPUT: IX -> Entidad
_sys_render_one_entity::
  
    ex de,hl                      ;; Guardamos entidad en HL
    ld c,l                        ;; / Guardamos entidad en BC
    ld b,h                        ;; \
    push bc

    ld de,#screen_start                 ;; lastVP = C000 (de momento)
    ld c,e_x(ix)                  ;; Entity X
    ld b,e_y(ix)                  ;; Entity Y
    call cpct_getScreenPtr_asm    ;; HL = pvmen

    ld a, e_estado(ix)
    cp #0x82
    jr z, e_muerta

        ex de, hl                     ;; DE = pvmen
        ld l, e_sprite-1(ix)          ;; Entity Sprite (pos 8 bytes)
        ld h, e_sprite+0(ix)          ;; Entity Sprite (pos 9 bytes)
        
        ld c, e_w(ix)                 ;; Entity Width
        ld b, e_h(ix)                 ;; Entity Height
        call cpct_drawSprite_asm    
        jr z, continue
    e_muerta:
        ex de, hl
        ld l, e_sprite-1(ix)
        ld h, e_sprite+0(ix)          
        xor a
        ld b, e_h(ix)                 ;; Entity Height
        ld c, e_w(ix)                 ;; Entity Width
        call cpct_drawSolidBox_asm  
        jr continue
    
    continue:
    pop bc                        ;; Recuperamos entidad de BC
    ld de,#_sys_render_one_entity ;; Volvemos a poner en el registro DE la posicion de memoria de la funcion
    push de
    jp _destroyed                 ;; Saltamos a _destroyed con entidad en BC
        
    
   
    ret

;; D = Registro
;; E = Valor
;;_sys_set_crtc:
;;    ld bc, #0xBC00               
;;    out (c), d                   
;;    inc b
;;    out (c), e
;;
;;    ret
_sys_change_screen_resolution::
    ;;DIRECTION 0ZBC => CRTC Registro
    ;;R1 = 32 (Ancho mostrado en caracteres)
    ;;ld de, #0x0120       ;; 01 -> Registro. 0x20 (32 decimal) -> valor
    ;;call _sys_set_crtc

    ;;;;R2 = 42 (Inicio HSYNC)
    ;;ld de, #0x022A       ;; 02 -> Registro. 0x2A -> valor
    ;;call _sys_set_crtc

    ;;R6 = 20 (Alto mostrado en caracteres)
    ;;ld de, #0x0614       ;; 06 -> Registro. 0x10 -> valor
    ;;call _sys_set_crtc

    ;;;;R7 = 26 (Comienzo VSYNC)
    ;;ld de, #0x071A       ;; 07 -> Registro. 0x1A -> valor
    ;;call _sys_set_crtc

    ;;R9 = 6 
    ;;ld de, #0x0906       ;; 09 -> Registro. 0x06 -> valor
    ;;call _sys_set_crtc

    ret
_sys_render_update::
    ld ix, #_entidades
    ld a, e_vida(ix)
    cp #0x00
    jr z, not_render
        call _sys_render_vida
        call _sys_render_player_score
        jr z, finish
    not_render:
    jr finish
    finish:
        ld c,#0x01
        ld de,#_sys_render_one_entity ;;Guardamos en el registro DE la posición de memoria de la funcion _sys_render_one_entity
        call _man_entity_forall

    ret




    